#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Fri Dec 22 20:11:24 2023

@auteur: David ALBERTO
(www.astrolabe-science.fr)
Ce script importe les heures de lever et de coucher du Soleil, depuis un fichier csv, calcule la durée
du jour et trace la différence de durée du jour, pour chaque jour, en secondes.
Le fichier externe est téléchargé depuis le formulaire de calculs d'éphémérides de l'IMCCE
(https://ssp.imcce.fr/forms/visibility).
"""
import matplotlib.pyplot as plt # graphique
import numpy as np # gestion de listes
import pandas as pd # import de données et gestion de tableaux de données
import datetime as dt # gestion de données au format date
import locale # noms des mois en francais
import matplotlib.patches as mpatches # pour paramétrer les flèches d'annotation
import matplotlib.image as image
from matplotlib.offsetbox import (OffsetImage, AnnotationBbox) # pour insertion d'une image (logo Creative Commons)

locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8') # dates en français

plt.rcParams["font.family"] = "Roboto Condensed" # ou autre police installée
plt.rcParams["font.size"] = 8 # taille des caractères
annee = 2024

# import des données depuis un fichier externe :
eph = pd.read_csv('ephSoleil2024.csv', sep=';',
                  usecols=[1, 2, 6],
 names = ['date', 'lever', 'coucher'],
 skiprows = 1,
 parse_dates = True,
 index_col ='date')

eph = eph[eph.index < dt.datetime(annee,12,22)]# limitation de la dernière date
dates = eph.index

def Nmois(date):
    """
    renvoie le numéro du mois, d'une date donnée (entier).
    """
    M = date.strftime("%m")
    M = int(M)
    return M

def dureejour(date):
    """
    À partir d'une date, renvoie la durée du jour en heures.
    """
    lever = eph.loc[dates == date, 'lever'].values[0]
    coucher = eph.loc[dates == date, 'coucher'].values[0]
    duree = coucher - lever
    return duree

def difference(N):
    """
    renvoie la différence de durée du jour entre un jour de rang N et le précédent, dans la liste de dates.
    """
    veille = dureejour(dates[N-1])
    ajd = dureejour(dates[N])
    diff = ajd - veille
    sec = diff * 3600
    return sec

def totalmois(M):
    precedent = dt.datetime(annee, M,1) - dt.timedelta(days=1)
    dernierjour = dt.datetime(annee, M+1,1) - dt.timedelta(days=1)
    lever1 = eph[precedent:precedent].lever[0]
    coucher1 = eph[precedent:precedent].coucher[0]
    lever2 = eph[dernierjour:dernierjour].lever[0]
    coucher2 = eph[dernierjour:dernierjour].coucher[0]
    ecart = coucher2 - lever2 - (coucher1 - lever1)
    ecart = ecart * 60 # en minutes
    ecart = round(ecart,1)
    if ecart > 0:
        ecart = "+" + str(ecart)
    else:ecart = str(ecart)
    ecart = ecart.replace(".", ",")
    axe.text(dernierjour, -207, f"{ecart} min",
             color = 'k', 
             ha='right',va = 'center',
             bbox=dict(facecolor=cmap(M), ec='None', alpha=0.5,
                       pad=0),)
    # return ecart

cmap = plt.get_cmap('tab20c') # définition du colormap

# création de la figure et de la fenêtre graphique :
fig = plt.figure(
    figsize=(29.7 / 2.54, 21 / 2.54),# format A4 horizontal
    tight_layout = True # marges réduites
    )

axe = plt.subplot(111)
axe.set(
        xlim=(dates[0], dates[-1]),
        xticks=[],# suppression des graduations
        ylim=(-210,210),
        yticks=np.arange(-240,240,60),
        ylabel="différence (secondes)",
        axisbelow = True,
        )

axe.set_yticks(np.arange(-210,210,10), minor = True)

axe.set_title("Variation quotidienne de la durée du jour (écart par rapport au jour précédent, en secondes)",
              fontsize=16, fontweight='bold')

diff = [difference(i) for i in range(len(dates))]# liste des différences
diff[0] = np.nan # suppression de l avaleur initiale, inexacte
# eph["diff"] = diff # ajout des valeurs de différences dans le tableau eph (optionnel)

# création d'une liste de couleurs par jour selon le n° du mois :
mois = [Nmois(i) for i in dates]# liste des numéros du mois, pour chaque jour
couleurs = [cmap(M) for M in mois] # liste de couleurs

# tracé de l'histogramme :
axe.bar(dates, diff, color = couleurs)

# affichage des mois :
for N in range(1,13):
    date = dt.datetime(2024,N,15)
    nommois = date.strftime('%b')
    if N < 7:
        y = -10
    else:y = 10
    couleur = cmap(Nmois(date))
    axe.text(date, y, nommois, fontsize=8, ha='center',va='center',
             bbox=dict(facecolor=couleur, ec='None', alpha=0.5))

axe.text(dates[0], -207," total du mois :", va = 'center')
for M in range(1,12):
    totalmois(M)

texte = "Données 2024. Latitude 47° nord.\nLes calculs de visibilité des astres ont été réalisés par le service de calcul des éphémérides\nde l’IMCCE à travers son portail Système solaire (https://ssp.imcce.fr) "

axe.text(0.01,0.01, texte, transform = axe.transAxes, ha='left', va='bottom',
         fontsize=8)

# annotations :
arr = mpatches.FancyArrowPatch((dt.datetime(2024,7,1),60), (dt.datetime(2024,6,20), 0),
                               arrowstyle='->,head_width=.15',
                                mutation_scale=20,
                                color=cmap(6),
                               )
axe.add_patch(arr)
axe.annotate("solstice d'été", (2.1, 1), xycoords=arr, ha='center', va='bottom',
             color = cmap(6))

arr = mpatches.FancyArrowPatch((dt.datetime(2024,1,1),-60), (dates[0], 0),
                               arrowstyle='->,head_width=.15',
                                mutation_scale=20,
                                color=cmap(12),
                               )
axe.add_patch(arr)
axe.annotate("solstice d'hiver\n2023", (2.4, 0), xycoords=arr, ha='center', va='top',
             color = cmap(12))

arr = mpatches.FancyArrowPatch((dt.datetime(2024,3,20),-60), (dt.datetime(2024,3,20), -20),
                               arrowstyle='->,head_width=.15',
                                mutation_scale=20,
                                color=cmap(3),
                               )
axe.add_patch(arr)
axe.annotate("équinoxe de printemps", (2.4, -0.2), xycoords=arr, ha='center', va='bottom',
             color = cmap(3))

arr = mpatches.FancyArrowPatch((dt.datetime(2024,9,22),60), (dt.datetime(2024,9,22), 20),
                               arrowstyle='->,head_width=.15',
                                mutation_scale=20,
                                color=cmap(9),
                               )
axe.add_patch(arr)
axe.annotate("équinoxe d'automne", (2.4, 1.0), xycoords=arr, ha='center', va='bottom',
             color = cmap(9))

# signature :
axe.text(0.999, 0.01, "D. Alberto (www.astrolabe-science.fr",
         rotation=90, ha='right',va='bottom', color='gray',
         transform = axe.transAxes,)

# insertion du logo CC :
logo = image.imread("by-nc-sa.png")
imagebox = OffsetImage(logo, zoom = 0.15)
ab = AnnotationBbox(imagebox, (dates[-20], -230), frameon = False)
axe.add_artist(ab)

fig.savefig("dureeJourDiffGraphe.pdf")
fig.savefig("dureeJourDiffGraphe.png", dpi=300)
